#converts a tree from matrix format to Newick:
#assumes that the edge order of tree is compatible with
#the partial order of the tree.
convert <- function(tree, p, var.names) {
  if (!is.tree(tree))
    stop("'tree' is not in suitable matrix format.")
  if (!is.vector(p) | length(p) != ncol(tree))
    stop("Sizes of 'tree' and 'p' are not compatible.")
  if (!is.vector(var.names))
    stop("'var.names' should be a vector.")
  nodes <- sort(unique(as.vector(tree)))
  for (j in 1:length(nodes)) 
    tree[which(tree==nodes[j])] <- j
  res <- as.list(c(var.names, rep("(", length(nodes)-length(var.names)))) #every node of the tree gets a string, the first ones are the leaves
  #works also for non-binary trees
  done <- rep(FALSE, length(nodes)) #the nodes whose string is completed
  done[1:length(var.names)] <- TRUE
  while (!all(done)) {
    waiting <- numeric()
    for (j in which(!done)) {
      if (all(done[tree[2,which(tree[1,]==j)]]))
        waiting <- c(waiting, j)
    }
    for (j in waiting) {
      edges <- which(tree[1,] == j)
      for (k in 1:length(edges)) {
        tmp  <- paste(res[[tree[2,edges[k]]]], ":", format(-log(p[edges[k]]), digits=3), sep="")
        res[[j]] <- paste(res[[j]], tmp, sep="")
        if (k == length(edges))
          res[[j]] <- paste(res[[j]], ")", sep="")
        else
          res[[j]] <- paste(res[[j]], ",", sep="")
      }
      done[j] <- TRUE
    }
  }
  parents <- match(tree[1,],tree[2,])
  root <- tree[1,which(is.na(parents))]
  return(res[[root]])
}
